@@ -1,18 +1,40 @@
 import traceback
 
+import ssl
+
 from pypsql_api.config.types import Session
 from pypsql_api.wire.actions_types import Names
 from pypsql_api.context import Context
 from pypsql_api.wire.back import SSLNo, AuthenticationCleartextPassword, ReadyForQuery, AuthenticationOk, \
-    EmptyQueryResponse, DataFrameRowDescription, DataFrameDataRows, CommandComplete
+    EmptyQueryResponse, DataFrameRowDescription, DataFrameDataRows, CommandComplete, SSLYes
+from pypsql_api.wire.bytes import ReadingIO, WritingIO
 from pypsql_api.wire.errors import NoData
 from pypsql_api.wire.front import SSLRequest, StartupMessage, Message, PasswordMessage
 
+cipher_list = "ECDH+ECDSA+AESGCM:" \
+ \
+              "ECDH+ECDSA+AES:" \
+              "ECDH+AESGCM:" \
+              "ECDH+AES:" \
+ \
+              "DH+ECDSA+AESGCM:" \
+              "DH+ECDSA+AES:" \
+              "DH+AESGCM:" \
+              "DH+AES:" \
+ \
+              "RSA+AESGCM:" \
+              "RSA+AESCBC:" \
+ \
+              "!aNULL:!MD5:!DSS:"
+
 
 def read_ssl_request(context: Context):
     ssl_request = SSLRequest.read(context.input)
 
-    return context.update_mem('ssl_request', ssl_request), Names.WRITE_SSL_NO
+    if context.key_file and context.cert_file:
+        return context.update_mem('ssl_request', ssl_request), Names.WRITE_SSL_YES
+    else:
+        return context.update_mem('ssl_request', ssl_request), Names.WRITE_SSL_NO
 
 
 def write_ssl_resp_no(context: Context):
@@ -21,6 +43,27 @@ def write_ssl_resp_no(context: Context):
     return context, Names.READ_STARTUP_MESSAGE
 
 
+def write_ssl_resp_yes(context: Context):
+    SSLYes().write(context.output)
+
+    # we need wrap the connection in a SSL connection
+    ssl_obj: ssl.SSLSocket = ssl.wrap_socket(sock=context.socket,
+                                             do_handshake_on_connect=False,
+                                             server_side=True,
+                                             certfile=context.cert_file,
+                                             keyfile=context.key_file,
+                                             ciphers=cipher_list)
+
+    ssl_obj.do_handshake()
+
+    context.mem['ssl_obj'] = ssl_obj
+
+    context.input = ReadingIO(ssl_obj)
+    context.output = WritingIO(ssl_obj)
+
+    return context, Names.READ_STARTUP_MESSAGE
+
+
 def read_startup_message(context: Context):
     try:
         startup_front = StartupMessage.read(context.input)
@@ -39,8 +82,8 @@ def read_startup_message(context: Context):
     except:
         print(f"Error udring startup read")
         traceback.print_exc()
-        print("continuing with plain text password")
-        return context.update_mem('session', session), Names.WRITE_PLAIN_TEXT_PASSWORD_REQUEST
+        raise
+
 
 def write_plain_text_password_request(context: Context):
     AuthenticationCleartextPassword().write(context.output)
