@@ -4,7 +4,7 @@ from typing import Callable, Set, Tuple, Any
 
 from pypsql_api.wire.actions import write_ssl_resp_no, read_startup_message, read_plain_text_password_request, \
     write_plain_text_password_request, read_ssl_request, write_ready_for_query, write_auth_ok, read_receive_command, \
-    write_empty_response, write_data_frame_response, read_receive_extended_command
+    write_empty_response, write_data_frame_response, read_receive_extended_command, write_ssl_resp_yes
 from pypsql_api.wire.actions_types import Names
 from pypsql_api.wire.query.extended_query import process_parse_command, process_bind_command, process_execute_command, \
     read_till_sync, write_extended_data_frame_response, write_extended_empty_response, describe_statement_or_portal
@@ -34,8 +34,10 @@ def move_state(context: Any, state: State) -> Tuple[Any, str]:
 
 
 processes = {
-    Names.READ_SSL_REQUEST: State(name=Names.READ_SSL_REQUEST, allowed={Names.WRITE_SSL_NO}, fn=read_ssl_request),
+    Names.READ_SSL_REQUEST: State(name=Names.READ_SSL_REQUEST, allowed={Names.WRITE_SSL_NO, Names.WRITE_SSL_YES}, fn=read_ssl_request),
     Names.WRITE_SSL_NO: State(name=Names.READ_SSL_REQUEST, allowed={Names.READ_STARTUP_MESSAGE}, fn=write_ssl_resp_no),
+    Names.WRITE_SSL_YES: State(name=Names.READ_SSL_REQUEST, allowed={Names.READ_STARTUP_MESSAGE}, fn=write_ssl_resp_yes),
+
     Names.READ_STARTUP_MESSAGE: State(name=Names.READ_STARTUP_MESSAGE,
                                       allowed={Names.WRITE_PLAIN_TEXT_PASSWORD_REQUEST, Names.CLOSE,
                                                Names.READ_STARTUP_MESSAGE}, fn=read_startup_message),
